.progress {
  background: #888;
  position: relative;
}
.progress .bar {
  color: #fff;
  text-indent: 50%;
}
.progress span {
  position: absolute;
  z-index: 20000;
  color: #fff;
  text-align: center;
  width: 100%;
}
select.small {
    line-height: 20px;
    height: 20px;
}
